
//  cptChar.cpp
//-------------------
// Ce programme l'occurence de certains caractères dans un texte
//
// Paramètres sur la ligne de commandes
//  - Parm #1 : nombre de processus à utiliser
//  - Parm #2 : nom du fichier contenant le texte
//  - Parm #3 : nom du fichier de sortie

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <CL/cl.h>
#include <random>
#include <iostream>
#include "contexte.h"
#include "program.h"
#include <sstream>
#include <chrono>

using namespace std;
using namespace std::chrono;

////////////////////////////////////////////////////////////////////
//  Constantes
//////////////////////////////////////////////////////////////
const int PROUST = 140620770;   // Taille du texte
const int PROUST1 = 47109830;   // Taille du texte
const int PROUST2 = 1346000;    // Taille du texte
const int TEXTE1 =5000;         // Taille du texte
const int T_RESUL = 64;         // Taille de l'alphabet
const int NB_COEURS = 62;            // Nombre de coeurs à utiliser sur la carte graphique
 // Alphabet à considérer
const char alpha[] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
////////////////////////////////////////////////////////////////////////////
// Tableau global pour les résultats
////////////////////////////////////////////////////////////////////////////
int nbCar[T_RESUL];
/********************************************************************************
   Programme principale qui initialise et démarre les fils
   
   Entree : (sur la ligne de commandes)
            nbFil - nombre de fils d'exécution
            nom : fichier contenant le texte
            nom_sortie : fichier contenat le résultat
    
********************************************************************************/
int main(int argc, char *argv[])
{
    int taille_texte = 0;
    const char* texte = 0; 
    string tmp;
    int carParFil[62];  // Pour emmagasiner les décompte des caractères
    int nbCoeurs;
    string nom,         // nom du fichier contanenat le texte
           nom_sortie;  // nom du fichier contenant le résultat
    ifstream manuel;    // Pour lire le texte à traiter
    ofstream cpt_res;   // Pour écrire le résultat en sortie
    
    // Obtention des arguments 
    nom = argv[1];          // cout<< nom << " -- " ;
    nom_sortie = argv[2];   // cout<< nom << endl ; 
    //-----------------------------------------------------------------
    // **************** Affichage pour mise au point
    // cout<< "Execution sur " << nbCoeurs << " coeurs de " << nom<< endl;
    //-----------------------------
    // Lecture du texte
    manuel.open(nom);
    if (manuel.is_open())
    {
      ostringstream ss;
      ss << manuel.rdbuf(); 
      tmp = ss.str();
      texte = tmp.c_str();
      manuel.close();
    }
    else cout << "Impossible d'ouvrir le fichier" << endl; 
    taille_texte = tmp.size();
    // Initialisation du tableau pour le décompte de caractères
    for(int i = 0; i < 64; i++)  {  nbCar[i] = 0 ;}
   
    //---------------------------------------------------------
    // Positionnement du chronomètre pour évaluer la performance
    auto temps_depart = system_clock::now();
   
    //===============================================
    // On initialise et execute le programme opencl
    Contexte mon_contexte;
    Program mon_prog(mon_contexte, "./kernel.cl", "cptCar");
    mon_prog.initPara(nbCar, T_RESUL, SORTIE, texte, taille_texte, ENTREE, alpha, 62, ENTREE, taille_texte);
    mon_prog.execute(1, NB_COEURS, 1, nbCar);
    //-------------------------------------------------
    // Arrêt du chronomètre
    auto temps_arret = system_clock::now();
    // -------------------------------------------------
    // Affichage du temps d'exécution
    auto duree = duration_cast<milliseconds>(temps_arret - temps_depart);
    cout << "Temps execution = " << duree.count()/1000.0 << " secondes" << endl;
    // -------------------------------------------------
    // Sauvegarde du résultat dans fichier de sortie
    cpt_res.open(nom_sortie,ios::binary | ios::out);
    if (cpt_res.is_open())
    {
        cpt_res << "Occurences des lettres"<< endl; 
        for (int i = 0; i< 62; i++)
        {  
            if ((i+1)%12 != 0) cpt_res << "(" << alpha[i] << ":" << nbCar[i]<< "), ";
            else cpt_res << "(" << alpha[i] << ":" << nbCar[i]<< ") " << endl;
            // Affichage à l'écran si désiré...
            //cout << "Nombre de " << alpha[i] << " = " << nbCar[i]<< endl;
        } 
    }  
}
